from datetime import datetime
from typing_extensions import Literal, NotRequired, TypedDict
from bit2me.types import DepositEstimation, FundsOrigin, Phone, UserType
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class AttorneyInFact(TypedDict):
  name: NotRequired[str]
  surname: NotRequired[str]
  typeOfDocument: NotRequired[Literal['none', 'idcard', 'foreign-id-card', 'passport', 'drivinglicense', 'visa']]
  numberOfDocument: NotRequired[str]

class Avatar(TypedDict):
  sm: NotRequired[str]
  md: NotRequired[str]
  lg: NotRequired[str]

class Contact(TypedDict):
  name: NotRequired[str]
  surname: NotRequired[str]
  typeOfDocument: NotRequired[Literal['none', 'idcard', 'foreign-id-card', 'passport', 'drivinglicense', 'visa']]
  numberOfDocument: NotRequired[str]

class Identity(TypedDict):
  type: NotRequired[Literal['none', 'idcard', 'passport', 'drivinglicense', 'visa', 'cif']]
  number: NotRequired[str]
  countryCode: NotRequired[str]
  updatedAt: NotRequired[datetime]

class PartnersItem(TypedDict):
  nationalityCountryCode: NotRequired[str]
  companyPartnerId: str
  documentType: NotRequired[Literal['none', 'idcard', 'foreign-id-card', 'passport', 'drivinglicense', 'visa']]
  documentNumber: NotRequired[str]
  documentCountryCode: NotRequired[str]
  birthdate: NotRequired[datetime]
  sharePercentage: NotRequired[float]
  fullName: NotRequired[str]
  isPoliticallyExposed: NotRequired[bool]
  deletedAt: NotRequired[datetime]

class Person(TypedDict):
  name: NotRequired[str]
  surname: NotRequired[str]
  gender: NotRequired[Literal['male', 'female', 'N/D']]
  nationalityCountryCode: NotRequired[str]

class PoliticalPartnersItem(TypedDict):
  name: NotRequired[str]
  position: NotRequired[str]

class SecondFactorAuth(TypedDict):
  alwaysRequired: NotRequired[bool]

class Company(TypedDict):
  businessName: NotRequired[str]
  kindOfSociety: NotRequired[str]
  constitutionDate: NotRequired[datetime]
  constitutionCountryCode: NotRequired[str]
  registeredOffice: NotRequired[str]
  sector: NotRequired[str]
  autonomus: NotRequired[bool]
  obligedSubject: NotRequired[bool]
  overTheCounter: NotRequired[bool]
  politicallyExposedPersonPartners: NotRequired[bool]
  politicalPartners: NotRequired[list[PoliticalPartnersItem]]
  firstTradeEstimatedValue: NotRequired[int]
  annualTradeEstimatedValue: NotRequired[int]
  billingVolume: NotRequired[int]
  numberOfPartners: NotRequired[int]
  reasonForTheOperation: NotRequired[list[str]]
  typeOfOperationsToPerform: NotRequired[list[str]]
  identity: NotRequired[Identity]
  taxId: NotRequired[str]
  legalEntityId: NotRequired[str]
  contact: NotRequired[Contact]
  attorneyInFact: NotRequired[AttorneyInFact]
  legalTerms: NotRequired[bool]
  service: NotRequired[str]
  webUrl: NotRequired[str | None]
  shareholdingStructure: NotRequired[str | None]
  services: NotRequired[list[Literal['crypto_investment', 'send_crypto', 'liquidity', 'earn_loan', 'api', 'consulting', 'other']]]
  otherServices: NotRequired[str | None]
  partners: NotRequired[list[PartnersItem]]
  economicCapacityModel: NotRequired[Literal['model_200', 'model_390', 'model_303']]

class Profile(TypedDict):
  langCode: NotRequired[str]
  currencyCode: NotRequired[str]
  timeZone: NotRequired[str]
  avatar: NotRequired[Avatar]

class AccountDetailsV3response(TypedDict):
  id: str
  email: str
  realEmail: NotRequired[str]
  phone: NotRequired[Phone]
  alias: NotRequired[str]
  person: NotRequired[Person]
  profile: NotRequired[Profile]
  selfInvoiceIdentityNumber: NotRequired[str]
  registrationDate: NotRequired[datetime]
  type: NotRequired[UserType]
  depositEstimation: NotRequired[DepositEstimation]
  fundsOrigin: NotRequired[FundsOrigin]
  accountPurpose: NotRequired[Literal['savings', 'currencyExchange', 'investment', 'trading', 'cryptocurrencyPayments']]
  secondFactorAuth: NotRequired[SecondFactorAuth]
  documentValidity: NotRequired[Literal['valid', 'almostExpired', 'expired', 'updateRequired']]
  company: NotRequired[Company]
  accountCompleted: NotRequired[bool]
  knowledgeLevel: NotRequired[float]
  registrationReason: NotRequired[Literal['investing', 'diversification', 'savings', 'curiosity', 'blockchain']]
  knowledgeQuestionsStatus: NotRequired[Literal['completed', 'uncompleted', 'omitted']]

adapter = TypeAdapter(AccountDetailsV3response)

class Get(Endpoint):
  async def get(self, *, validate: bool = True) -> AccountDetailsV3response:
    """Get account details, personal data is returned masked
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/GET/v3/account)"""
    r = await self.authed_request('GET', '/v3/account')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class EarnUserSummaryResponse(TypedDict):
  currency: str
  totalBalance: float
  totalRewards: float

adapter = TypeAdapter(EarnUserSummaryResponse)

class Summary(Endpoint):
  async def __call__(
    self,
    *,
    user_currency: str | None = None,
    validate: bool = True
  ) -> EarnUserSummaryResponse:
    """Retrieves user summary
    
    - `user_currency`: Currency to show balance
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v1/earn/summary)"""
    params = {}
    if user_currency is not None:
      params['userCurrency'] = user_currency
    r = await self.authed_request('GET', '/v1/earn/summary', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from bit2me.types import Phone
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class CreateSubaccountResponse(TypedDict):
  userId: str

class CreateSubaccountRequest(TypedDict):
  email: str
  name: str
  surname: str
  phone: Phone

adapter = TypeAdapter(CreateSubaccountResponse)

class Create(Endpoint):
  async def __call__(
    self,
    create_subaccount_request: CreateSubaccountRequest,
    *,
    validate: bool = True
  ) -> CreateSubaccountResponse:
    """Create a subaccount.
    
    - `create_subaccount_request`: The details to create the subaccount
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/POST/v1/account/subaccount)"""
    r = await self.authed_request(
      'POST', '/v1/account/subaccount',
      json=create_subaccount_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from bit2me.types import LoanAction
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class PaybackGuaranteeRequest(TypedDict):
  paybackAmount: str

adapter = TypeAdapter(LoanAction)

class Payback(Endpoint):
  async def payback(
    self,
    order_id: str,
    payback_guarantee_request: PaybackGuaranteeRequest,
    *,
    validate: bool = True
  ) -> LoanAction:
    """Process a payment to refund a specified amount of an loan. This action will complete the loan if the outstanding amount is fully paid.
    
    - `order_id`
    - `payback_guarantee_request`: data
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/loan/POST/v1/loan/orders/{orderId}/payback)"""
    r = await self.authed_request(
      'POST', f'/v1/loan/orders/{order_id}/payback',
      json=payback_guarantee_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(BooleanResultResponse)

class Ibans(Endpoint):
  async def __call__(self, *, iban: str, validate: bool = True) -> BooleanResultResponse:
    """Check whether an IBAN is valid.
    
    - `iban`: Iban to check
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/funding/GET/v1/teller/iban/validate)"""
    params: dict = {
      'iban': iban,
    }
    r = await self.authed_request('GET', '/v1/teller/iban/validate', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Literal, NotRequired, TypedDict
from bit2me.types import (
  BooleanResultResponse,
  DepositEstimation,
  FundsOrigin,
  UserType
)
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Address(TypedDict):
  country: str
  """The 2-letter country code"""
  state: str
  city: str
  street: str
  zip: str

class Identity(TypedDict):
  type: NotRequired[Literal['idcard', 'passport', 'drivinglicense', 'visa']]
  number: NotRequired[str]
  expiryDate: NotRequired[datetime]
  countryCode: NotRequired[str]

class Profile(TypedDict):
  langCode: NotRequired[str]
  currencyCode: NotRequired[str]
  timeZone: NotRequired[str]

class Person(TypedDict):
  name: NotRequired[str]
  surname: NotRequired[str]
  gender: NotRequired[Literal['male', 'female', 'N/D']]
  birthDate: NotRequired[datetime]
  nationalityCountryCode: NotRequired[str]
  employmentStatus: NotRequired[Literal['employed', 'student', 'selfEmployed', 'retired', 'unemployed']]
  profession: NotRequired[str]
  identity: NotRequired[Identity]

class UpdateAccountRequest(TypedDict):
  alias: NotRequired[str]
  person: NotRequired[Person]
  profile: NotRequired[Profile]
  address: NotRequired[Address]
  depositEstimation: NotRequired[DepositEstimation]
  selfInvoiceIdentityNumber: NotRequired[str]
  fundsOrigin: NotRequired[FundsOrigin]
  type: NotRequired[UserType]
  so: NotRequired[str]
  version: NotRequired[str]

adapter = TypeAdapter(BooleanResultResponse)

class Update(Endpoint):
  async def __call__(
    self,
    update_account_request: UpdateAccountRequest,
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """Update account information.
    
    - `update_account_request`: The account details to be updated
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/PUT/v1/account/update)"""
    r = await self.authed_request(
      'put', '/v1/account/update',
      json=update_account_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from bit2me.types import WalletResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class TradingWalletDepositRequest(TypedDict):
  fromPocketId: str
  """Bit2Me pocket identifier"""
  amount: str
  currency: str
  """Valid currency symbol"""

adapter = TypeAdapter(WalletResponse)

class RequestDeposit(Endpoint):
  async def request_deposit(
    self,
    trading_wallet_deposit_request: TradingWalletDepositRequest,
    *,
    validate: bool = True
  ) -> WalletResponse:
    """Request deposit funds into your Pro account from a Bit2Me Wallet.
    
    - `trading_wallet_deposit_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/funding/POST/v1/trading/wallet/deposit)"""
    r = await self.authed_request(
      'POST', '/v1/trading/wallet/deposit',
      json=trading_wallet_deposit_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Any, Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Amount(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]

class DataItem(TypedDict):
  movementId: NotRequired[str]
  userId: NotRequired[str]
  type: NotRequired[Literal['deposit', 'withdrawal', 'reward']]
  status: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]
  walletId: NotRequired[str]
  lockId: NotRequired[str | None]
  amount: NotRequired[Amount]
  withholdingAmount: NotRequired[dict[str, Any]]
  netAmount: NotRequired[dict[str, Any]]
  rate: NotRequired[dict[str, Any]]
  convertedAmount: NotRequired[dict[str, Any]]
  source: NotRequired[dict[str, Any] | None]
  issuer: NotRequired[dict[str, Any]]

class ListEarnWalletMovementsResponse(TypedDict):
  total: int
  """Total movements matching the query"""
  data: list[DataItem]
  """Earn movements returned"""

adapter = TypeAdapter(ListEarnWalletMovementsResponse)

class ListMovements(Endpoint):
  async def list_movements(
    self,
    wallet_id: str,
    *,
    user_currency: str | None = None,
    offset: int | None = None,
    limit: int | None = None,
    sort_by: Literal['createdAt'] | None = None,
    sort_direction: Literal['ascending', 'descending'] | None = None,
    validate: bool = True
  ) -> ListEarnWalletMovementsResponse:
    """Retrieve the wallet movement list.
    
    Movements can be paginated with `offset` and `limit`. For example, to fetch the third page with 20 records per page:
    
    ```text
    /v1/earn/wallets/{walletId}/movements?offset=40&limit=20
    ```
    
    - `wallet_id`: Wallet identifier
    - `user_currency`: Currency to show convertedAmount
    - `offset`: Specify the number of entries to be skipped (0 by default)
    - `limit`: Specify the maximum number of entries to be returned (20 by default)
    - `sort_by`: Sorting field
    - `sort_direction`: Sorting direction
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v1/earn/wallets/{walletId}/movements)"""
    params = {}
    if user_currency is not None:
      params['userCurrency'] = user_currency
    if offset is not None:
      params['offset'] = offset
    if limit is not None:
      params['limit'] = limit
    if sort_by is not None:
      params['sortBy'] = sort_by
    if sort_direction is not None:
      params['sortDirection'] = sort_direction
    r = await self.authed_request(
      'GET', f'/v1/earn/wallets/{wallet_id}/movements',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class LoanOrderResponse(TypedDict):
  orderId: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]
  userId: NotRequired[str]
  requestedByUserId: NotRequired[str]
  guaranteeCurrency: NotRequired[str]
  guaranteeOriginalAmount: NotRequired[str]
  guaranteeAmount: NotRequired[str]
  loanCurrency: NotRequired[str]
  loanOriginalAmount: NotRequired[str]
  loanAmount: NotRequired[str]
  paybackAmount: NotRequired[str]
  ltv: NotRequired[str]
  apr: NotRequired[str]
  status: NotRequired[str]
  startedAt: NotRequired[datetime]
  finishedAt: NotRequired[datetime]
  expiresAt: NotRequired[datetime]
  paybackPrincipalAmount: NotRequired[str]
  paybackInterestAmount: NotRequired[str]
  interestAmount: NotRequired[str]
  loanAmountConverted: NotRequired[str]
  paybackAmountConverted: NotRequired[str]
  guaranteeAmountConverted: NotRequired[str]
  remainingAmount: NotRequired[str]
  remainingAmountConverted: NotRequired[str]
  remainingPrincipalAmount: NotRequired[str]
  remainingInterestAmount: NotRequired[str]
  liquidationPriceReference: NotRequired[str]
  discount: NotRequired[str]
  isUserApprovalRequired: NotRequired[bool]

adapter = TypeAdapter(LoanOrderResponse)

class Get(Endpoint):
  async def get(self, order_id: str, *, validate: bool = True) -> LoanOrderResponse:
    """Get a user loan order
    
    - `order_id`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/loan/GET/v1/loan/orders/{orderId})"""
    r = await self.authed_request('GET', f'/v1/loan/orders/{order_id}')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import WalletResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class TradingWalletWithdrawalRequest(TypedDict):
  toPocketId: NotRequired[str]
  """Bit2Me pocket identifier"""
  amount: str
  currency: str
  """Valid currency symbol"""

adapter = TypeAdapter(WalletResponse)

class RequestWithdrawal(Endpoint):
  async def request_withdrawal(
    self,
    trading_wallet_withdrawal_request: TradingWalletWithdrawalRequest,
    *,
    validate: bool = True
  ) -> WalletResponse:
    """Request a withdrawal from the Pro account to the Bit2Me wallet.
    
    - `trading_wallet_withdrawal_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/funding/POST/v1/trading/wallet/withdraw)"""
    r = await self.authed_request(
      'POST', '/v1/trading/wallet/withdraw',
      json=trading_wallet_withdrawal_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()
